#
rm(list=ls())

#
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/feature_selection/permutation_importance")

#charger les donnes
library(xlsx)
D <- read.xlsx("breast_rnd.xlsx",header = T,sheetIndex = 1)
print(summary(D))

#partition des donnes en TRAIN et TEST
set.seed(100)
id <- sample(1:nrow(D),499,replace = FALSE)
DTrain <- D[id,]
DTest <- D[-id,]

#variables X : TRAIN et TEST
XTrain <- DTrain[2:ncol(DTrain)]
XTest <- DTest[2:ncol(DTest)]

#rgression logistique
modele <- glm(classe ~ ., data = DTrain, family = binomial)
print(summary(modele))

#coefficients standardiss
cstd <- modele$coefficients[2:ncol(DTrain)]*sapply(XTrain,sd)
print(cstd)

#reprsentation graphique
par(las=2)
barplot(sort(abs(cstd)),horiz=TRUE,cex.names=0.7,col="orange")

#calcul de l'erreur en prediction
err_pred <- function(X,y,modele){
  p <- factor(ifelse(predict(modele,newdata=X,type="response") > 0.5, "malignant", "begnin"))
  err <- mean(as.numeric(y != p))  
  return(err)
}

#erreur de rfrence en train
err_ref_train <- err_pred(XTrain,DTrain$classe,modele)
print(err_ref_train)

#calculer le ratio pour un ensemble de variables
#permutation feature importance
calc_ratio <- function(X,y,err_orig,modele){
  #traiter une colonne j
  calc_for_j <- function(j){
    Z <- X
    Z[,j] <- sample(X[,j])
    err <- err_pred(Z,y,modele)
    return(err/err_orig)
  }
  #pour l'ensemble des colonnes
  ratios <- sapply(1:ncol(XTest),calc_for_j)
  return(ratios)
}

#un essai sur le train set
print(calc_ratio(XTrain,DTrain$classe,err_ref_train,modele))

#fonction pour repliquer n_rep fois l'exprimentation
feature_importance <- function(modele, X, y, err_orig, n_rep){
  #rpeter les calculs n_rep fois
  few_ratios <- replicate(n_rep,calc_ratio(X,y,err_orig,modele))
  #nommer les lignes et les colonnes de la matrice
  rownames(few_ratios) <- colnames(X)
  colnames(few_ratios) <- 1:n_rep
  #
  return(few_ratios)
}

#repliquer 200 fois
set.seed(1)
results_train <- feature_importance(modele,XTrain,DTrain$classe, err_ref_train,n_rep=200)

#mdianes
med_train <- apply(results_train,1,median)

#reprsentation graphique
barplot(sort(med_train),horiz=TRUE,cex.names=0.7,col="lightskyblue",main="TRAIN SET",xlab="Median of feature importance")

#si on passe sur des box plot
index <- order(med_train)
boxplot(t(results_train[index,]),horizontal=TRUE,pars=list(cex.axis=0.7),main="TRAIN SET",xlab="Boxplot of feature importance")

#calcul de l'erreur en test
err_ref_test <- err_pred(XTest,DTest$classe,modele)
print(err_ref_test)

#utilisation de l'chantillon test
set.seed(1)
results_test <- feature_importance(modele,XTest,DTest$classe, err_ref_test,n_rep=200)

#mdianes
med_test <- apply(results_test,1,median)

#reprsentation graphique
barplot(sort(med_test),horiz=TRUE,cex.names=0.7,col="greenyellow",main="TEST SET",xlab="Median of feature importance")

#si on passe sur des box plot
index <- order(med_test)
boxplot(t(results_test[index,]),horizontal=TRUE,pars=list(cex.axis=0.7),main="TEST SET",xlab="Boxplot of feature importance")

#comparaison de feature importance en TRAIN et TEST
plot(med_train,type="b",col="blue",xlab="",ylab ="Mean Ratio",xaxt="n")
axis(side=1,at=1:length(med_train),labels=names(med_train),cex.axis=0.7)
lines(med_test,type="b",col="green")
legend(13,3,legend=c("TRAIN","TEST"),col=c("blue","green"),lty=1)

#mlr pour instancier la rgression logistique autrement
#propose des prototypes standardiss de modlisation
#et d'accs aux proprits des modles
#un peu  la manire de "caret"
library(mlr)

#instancier le modle
lrn <- makeLearner("classif.logreg",predict.type = "prob")
print(lrn)

#dfinir la tche et les donnes  traiter
tache <- makeClassifTask(data=DTrain,target="classe")
print(tache)

#modlisation - apprentissage
classifieur <- train(lrn,tache)
print(classifieur)

#afficher le modle
print(getLearnerModel(classifieur))

#package mesurer l'importance du modle
library(iml)

#objet de modelisation dfini sur le TRAIN set
objetTrain <- Predictor$new(classifieur,data=XTrain,y=DTrain$classe)

#calculer l'importance
importanceTrain <- FeatureImp$new(objetTrain,loss="ce",n.repetitions = 200)
print(importanceTrain)

#erreur de rfrence utilise
print(importanceTrain$original.error)

#dtail des rsultats
print(importanceTrain$results)


#affichage
plot(importanceTrain)

#objet de modelisation dfini sur le test set
objetTest <- Predictor$new(classifieur,data=XTest,y=DTest$classe)

#calculer l'importance
importanceTest <- FeatureImp$new(objetTest,loss="ce",n.repetitions = 200)

#affichage
plot(importanceTest)

importanceTest
